/* aptTables.js */

"use strict";

/* --------------------------------------------------------------
   drawBea (ctx, {x,y})
   ----------------------------------------------------------------*/
function drawBea(ctx, ndb) {
    ctx.fillStyle = '#00ff00';
    ctx.beginPath(); ctx.arc(ndb.x, ndb.y, 3, -Math.PI/2, Math.PI/2); ctx.fill();
    ctx.fillStyle = '#ffffff';
    ctx.beginPath(); ctx.arc(ndb.x, ndb.y, 3,  Math.PI/2,-Math.PI/2); ctx.fill();
} // drawBea

/* --------------------------------------------------------------
   drawNDB (ctx, {x,y})
   ----------------------------------------------------------------*/
function drawNDB(ctx, ndb) {
    ctx.beginPath(); ctx.arc(ndb.x, ndb.y, 3, 0,2*Math.PI); ctx.stroke();
    ctx.beginPath(); ctx.arc(ndb.x, ndb.y, 6, 0,2*Math.PI); ctx.stroke();
    ctx.beginPath(); ctx.arc(ndb.x, ndb.y, 9, 0,2*Math.PI); ctx.stroke();
} // drawNDB

/* --------------------------------------------------------------
   drawRwy (ctx, rwy: {name1,name2,x,y,heading,lenPx,widPx})
   ----------------------------------------------------------------*/
function drawRwy(ctx,rwy) {
    ctx.save();  // SP 0 (default)
    ctx.translate(rwy.x,rwy.y);
    ctx.rotate(rwy.heading*Math.PI/180);  // rotate axes clockwise

    ctx.fillStyle = '#999999';
    ctx.fillRect(-rwy.widPx/2,-rwy.lenPx/2, rwy.widPx,rwy.lenPx);
    ctx.strokeStyle = '#ffffff';
    ctx.moveTo(0,-rwy.lenPx/2); ctx.lineTo(0,rwy.lenPx/2); ctx.stroke();

    ctx.font="15px Arial";
    ctx.fillStyle = '#000000';
    ctx.textAlign = 'center';
    ctx.save(); // SP 1 (axes aligned with rwy, and centred at middle of rwy)
    ctx.translate(0, rwy.lenPx/2+15);
    ctx.fillText(rwy.name1, 0, 0);
    ctx.restore(); // SP 1

    ctx.save(); // SP 1 (axes aligned with rwy, and centred at middle of rwy)
    ctx.translate(0,-rwy.lenPx/2-15);
    ctx.rotate(Math.PI);
    ctx.fillText(rwy.name2, 0,0);
    ctx.restore(); // SP 1

    ctx.restore() // SP 0 (default)
} // drawRwy

/* --------------------------------------------------------------
   drawPad (ctx, pad: {name,x,y,orient,lenPx,widPx})
   ----------------------------------------------------------------*/
function drawPad(ctx,pad) {
    ctx.save();  // SP 0 (default)
    ctx.translate(pad.x,pad.y);
    ctx.rotate(pad.orient*Math.PI/180);  // rotate axes clockwise

    ctx.fillStyle = '#999999';
    ctx.fillRect(-pad.widPx/2,-pad.lenPx/2, pad.widPx,pad.lenPx);
    ctx.strokeStyle = '#ffffff';
    ctx.moveTo(0,-pad.lenPx/2); ctx.lineTo(0,pad.lenPx/2); ctx.stroke();
    ctx.beginPath(); ctx.arc(0, 0, 7, 0,2*Math.PI); ctx.stroke();

    ctx.font="15px Arial";
    ctx.fillStyle = '#000000';
    ctx.textAlign = 'center';
    ctx.save(); // SP 1 (axes aligned with pad, and centred at middle of pad)
    ctx.translate(0, pad.lenPx/2+15);
    ctx.fillText(pad.name, 0, 0);
    ctx.restore(); // SP 1

    ctx.restore() // SP 0 (default)
} // drawPad

/* --------------------------------------------------------------
   drawApt (icao)
   ----------------------------------------------------------------*/
function drawApt(icao) {
    var canvases = document.getElementsByTagName("canvas");
    var canvas;
    for(var i=0; i<canvases.length; i++) {
        canvas = canvases[i];
        if(canvas.name==icao) break;
    }
    
    /*var canvas = document.querySelector('canvas#MAAB');*/
    /*var canvas = document.getElementById("MAAB");*/
    /*var  canvases = document.getElementsByTagName("canvas");*/
    /*debug('Searching for #id=\"'+icao+'\" canvas');*/
    /*var canvas = document.querySelector('#id=\"'+icao+'\" canvas');*/
    /*var canvas = document.querySelector('canvas[name='+icao+']');*/
    if(canvas==null) debug("Failed  to look up canvas[id="+icao+"]");

    var ctx = canvas.getContext('2d');
    ctx.fillStyle = '#ffcc66';
    ctx.fillRect(0,0,canvas.width,canvas.height);
    ctx.translate(canvas.width/2,canvas.height/2);

    // Draw tower
    ctx.fillStyle = '#000000';
    ctx.moveTo(0,-5); ctx.lineTo(-4,4);  ctx.lineTo(4,4); ctx.lineTo(0,-5); ctx.fill();
    
    var scale = 0.025;     // pix/m

    // Draw beacon and NDB
    var loc = apts[icao].loc;
    var bea = apts[icao].bea;
    var beaD = loc.p.rhumbDistanceTo(bea.p,radiusM); // metres
    var beaB = loc.p.rhumbBearingTo(bea.p); //deg
    drawBea(ctx, {x:scale*beaD*Math.sin(beaB*Math.PI/180),
                  y:-scale*beaD*Math.cos(beaB*Math.PI/180) });
    var ndb = apts[icao].ndb;
    var ndbD = loc.p.rhumbDistanceTo(ndb.p,radiusM); // metres
    var ndbB = loc.p.rhumbBearingTo(ndb.p); //deg
    drawNDB(ctx, {x:scale*ndbD*Math.sin(ndbB*Math.PI/180),
                  y:-scale*ndbD*Math.cos(ndbB*Math.PI/180) });

    // Draw runways
    if(apts[icao].rwys != null && apts[icao].rwys.length>0) {
        for(var rwyN=0; rwyN < apts[icao].rwys.length; rwyN++) {
            var rwy = apts[icao].rwys[rwyN];
            var rwyD = loc.p.rhumbDistanceTo(rwy.mid,radiusM); // metres
            var rwyB = loc.p.rhumbBearingTo(rwy.mid); //deg
            var x = scale*rwyD*Math.sin(rwyB*Math.PI/180);
            var y = -scale*rwyD*Math.cos(rwyB*Math.PI/180);
            drawRwy(ctx, {name1:rwy.rwyNo1, name2:rwy.rwyNo2, 
                          x:x,y:y, heading:rwy.heading, 
                          lenPx:scale*rwy.lengthM, 
                          widPx:6*scale*rwy.widthM});
        }
    }

    // Draw pads
    if(apts[icao].pad != null && apts[icao].pad.length>0) {
        for(var padN=0; padN < apts[icao].pad.length; padN++) {
            var pad = apts[icao].pad[padN];
            var padD = loc.p.rhumbDistanceTo(pad.p,radiusM); // metres
            var padB = loc.p.rhumbBearingTo(pad.p); //deg
            var x = scale*padD*Math.sin(padB*Math.PI/180);
            var y = -scale*padD*Math.cos(padB*Math.PI/180);
            drawPad(ctx, {name:pad.rwyNo,
                          x:x,y:y, orient:pad.orient, 
                          lenPx:3*scale*pad.lengthM, 
                          widPx:3*scale*pad.widthM});
        }
    }
} // drawApt

/* --------------------------------------------------------------
   makeAptTable (icao)
   ----------------------------------------------------------------*/
function makeAptTable(icao) {
    var aptTable = document.createElement("table");
    aptTable.className = "apt"; aptTable.title="Data for "+icao;
    var tr, td1, td2, td3, td4, td5, td6;

    // Row "Loc 1"
    tr = document.createElement("tr");
    aptTable.appendChild(tr);
    //  "Loc:"    "Elev"    "Lat"    "Lon"
    td1 = document.createElement("td");
    td1.innerHTML='Loc:';
    td2 = document.createElement("td");
    td2.className = "sh";
    td2.innerHTML='Elev';
    td3 = document.createElement("td");
    td3.className = "sh";
    td3.innerHTML='Lat';
    td4 = document.createElement("td");
    td4.className = "sh";
    td4.innerHTML='Lon';
    tr.appendChild(td1);
    tr.appendChild(td2);
    tr.appendChild(td3);
    tr.appendChild(td4);

    // Row "Loc 2"
    tr = document.createElement("tr");
    aptTable.appendChild(tr);
    // ""    "17000'"    "-15.253°"    "+34.816°"
    td1 = document.createElement("td");
    td1.innerHTML = "";
    td2 = document.createElement("td");
    td2.innerHTML = apts[icao].loc.elev+"'";
    td3 = document.createElement("td");
    td3.innerHTML = Dms.toLat(apts[icao].loc.p.lat,"d");
    td4 = document.createElement("td");
    td4.innerHTML = Dms.toLon(apts[icao].loc.p.lon,"d");
    tr.appendChild(td1);
    tr.appendChild(td2);
    tr.appendChild(td3);
    tr.appendChild(td4);

    // Row "NDB 1"
    tr = document.createElement("tr");
    aptTable.appendChild(tr);
    //  "NDB:"    "Name"    "Freq"    "Range"
    td1 = document.createElement("td");
    td1.innerHTML='NDB:';
    td2 = document.createElement("td");
    td2.className = "sh";
    td2.innerHTML='Name';
    td3 = document.createElement("td");
    td3.className = "sh";
    td3.innerHTML='Freq';
    td4 = document.createElement("td");
    td4.className = "sh";
    td4.innerHTML='Range';
    tr.appendChild(td1);
    tr.appendChild(td2);
    tr.appendChild(td3);
    tr.appendChild(td4);

    // Row "NDB 2"
    tr = document.createElement("tr");
    aptTable.appendChild(tr);
    // ""    "AB"    "375 kHz"    "150 Nm"
    td1 = document.createElement("td");
    td1.innerHTML = "";
    td2 = document.createElement("td");
    td2.innerHTML = apts[icao].ndb.name;
    td3 = document.createElement("td");
    td3.innerHTML = apts[icao].ndb.kHz+" kHz";
    td4 = document.createElement("td");
    td4.innerHTML = apts[icao].ndb.range+" Nm";
    tr.appendChild(td1);
    tr.appendChild(td2);
    tr.appendChild(td3);
    tr.appendChild(td4);

    // Row "Rwy 1"
    if(apts[icao].rwys != null && apts[icao].rwys.length>0) {
        tr = document.createElement("tr");
        aptTable.appendChild(tr);
        //  "Rwy:"    "Name"    "Dimensions"    "Heading"
        td1 = document.createElement("td");
        td1.innerHTML='Rwy:';
        td2 = document.createElement("td");
        td2.className = "sh";
        td2.innerHTML='Name';
        td3 = document.createElement("td");
        td3.className = "sh";
        td3.innerHTML='Dimensions';
        td4 = document.createElement("td");
        td4.className = "sh";
        td4.innerHTML='Heading';
        tr.appendChild(td1);
        tr.appendChild(td2);
        tr.appendChild(td3);
        tr.appendChild(td4);
    }

    // Row "Rwy 2"
    if(apts[icao].rwys != null && apts[icao].rwys.length>0) {
        for(var rwyN=0; rwyN<apts[icao].rwys.length; rwyN++) {
            tr = document.createElement("tr");
            aptTable.appendChild(tr);
            // ""    "14/32"    "4827x61 m"    "142°/322°"
            td1 = document.createElement("td");
            td1.innerHTML = "";
            td2 = document.createElement("td");
            td2.innerHTML = apts[icao].rwys[rwyN].rwyNo1+"/"+apts[icao].rwys[rwyN].rwyNo2;
            td3 = document.createElement("td");
            td3.innerHTML = apts[icao].rwys[rwyN].lengthM.toFixed(0)+" x "+apts[icao].rwys[rwyN].widthM.toFixed(0)+" m";
            td4 = document.createElement("td");
            td4.innerHTML = apts[icao].rwys[rwyN].heading.toFixed(0)+"°/"+(apts[icao].rwys[rwyN].heading+180).toFixed(0)+"°";
            tr.appendChild(td1);
            tr.appendChild(td2);
            tr.appendChild(td3);
            tr.appendChild(td4);
        }}

    // Row "ILS 1"
    if(apts[icao].ils != null && apts[icao].ils.length>0) {
        tr = document.createElement("tr");
        aptTable.appendChild(tr);
        //  "ILS:"    "Name"    "Freq"    "Range"
        td1 = document.createElement("td");
        td1.innerHTML='ILS:';
        td2 = document.createElement("td");
        td2.className = "sh";
        td2.innerHTML='Name';
        td3 = document.createElement("td");
        td3.className = "sh";
        td3.innerHTML='Freq';
        td4 = document.createElement("td");
        td4.className = "sh";
        td4.innerHTML='Range';
        td5 = document.createElement("td");
        td5.className = "sh";
        td5.innerHTML='Bearing';
        td6 = document.createElement("td");
        td6.className = "sh";
        td6.innerHTML='GS';
        tr.appendChild(td1);
        tr.appendChild(td2);
        tr.appendChild(td3);
        tr.appendChild(td4);
        tr.appendChild(td5);
        tr.appendChild(td6);
    }

    // Row "ILS 2"
    if(apts[icao].ils != null && apts[icao].ils.length>0) {
        for(var ilsN=0; ilsN<apts[icao].ils.length; ilsN++) {
            tr = document.createElement("tr");
            aptTable.appendChild(tr);
            // ""    "14 IMAA"    "108.10 MHz"    "25 Nm"    "140.0"    "3.0°"
            td1 = document.createElement("td");
            td1.innerHTML = "";
            td2 = document.createElement("td");
            td2.innerHTML = apts[icao].ils[ilsN].rwyNo+" "+apts[icao].ils[ilsN].name;
            td3 = document.createElement("td");
            td3.innerHTML = apts[icao].ils[ilsN].mHz.toFixed(2)+" MHz";
            td4 = document.createElement("td");
            td4.innerHTML = apts[icao].ils[ilsN].range.toFixed(0)+" Nm";
            td5 = document.createElement("td");
            td5.innerHTML = apts[icao].ils[ilsN].bearing.toFixed(1)+"°";
            td6 = document.createElement("td");
            td6.innerHTML = apts[icao].ils[ilsN].gs.toFixed(1)+"°";
            tr.appendChild(td1);
            tr.appendChild(td2);
            tr.appendChild(td3);
            tr.appendChild(td4);
            tr.appendChild(td5);
            tr.appendChild(td6);
        }}

    // Row "Pad 1"
    if(apts[icao].pad != null && apts[icao].pad.length>0) {
        tr = document.createElement("tr");
        aptTable.appendChild(tr);
        //  "Pad:"    "Name"    "Dimensions"    "Orientation"
        td1 = document.createElement("td");
        td1.innerHTML='Pad:';
        td2 = document.createElement("td");
        td2.className = "sh";
        td2.innerHTML='Name';
        td3 = document.createElement("td");
        td3.className = "sh";
        td3.innerHTML='Dimensions';
        td4 = document.createElement("td");
        td4.className = "sh";
        td4.innerHTML='Orientation';
        tr.appendChild(td1);
        tr.appendChild(td2);
        tr.appendChild(td3);
        tr.appendChild(td4);
    }

    // Row "Pad 2"
    if(apts[icao].pad != null && apts[icao].pad.length>0) {
        for(var padN=0; padN<apts[icao].pad.length; padN++) {
            tr = document.createElement("tr");
            aptTable.appendChild(tr);
            // ""    "H1"    "61x61 m"    "142°"
            td1 = document.createElement("td");
            td1.innerHTML = "";
            td2 = document.createElement("td");
            td2.innerHTML = apts[icao].pad[padN].rwyNo;
            td3 = document.createElement("td");
            td3.innerHTML = apts[icao].pad[padN].lengthM.toFixed(0)+" x "+apts[icao].pad[padN].widthM.toFixed(0)+" m";
            td4 = document.createElement("td");
            td4.innerHTML = apts[icao].pad[padN].orient.toFixed(0)+"°";
            tr.appendChild(td1);
            tr.appendChild(td2);
            tr.appendChild(td3);
            tr.appendChild(td4);
        }}

    return aptTable;
}  // makeAptTable

/* --------------------------------------------------------------
   addApt (icao)
   ----------------------------------------------------------------*/
function addApt(icao) {
    var box = document.getElementById("aptCheckbox");
    var divHideable = document.createElement("div");
    divHideable.id = icao;
    divHideable.className = (box.checked)? "revealed": "hidden";
    // child[0]
    var spanShow = document.createElement("span");
    spanShow.className = "showAlways";
    spanShow.innerHTML = "<span class='highlightedIcao'>"+apts[icao].icao+"</span> &nbsp;&nbsp;&nbsp;"+apts[icao].fullname;
    // child[1]
    var para = document.createElement("p");
    var aptTable = makeAptTable(icao);          // Table
    para.appendChild(aptTable);
    // child[2]
    var figure = document.createElement("figure");
    figure.className = "apt";
    var canvas = document.createElement("canvas");
    canvas.name = icao;
    canvas.width = "200";
    canvas.height = "200";
    canvas.title = apts[icao].fullname;
    var script = document.createElement("script");
    script.innerHTML = "drawApt('"+icao+"')";   // Graphic
    figure.appendChild(canvas);
    figure.appendChild(script);
    // child[3]
    var divClear = document.createElement("div");
    divClear.style.clear = "both";

    var aptDataSection = document.querySelector('div#airportData');
    aptDataSection.appendChild(divHideable);
    divHideable.appendChild(spanShow);
    divHideable.appendChild(para);
    divHideable.appendChild(figure);
    divHideable.appendChild(divClear);
} // addApt

/* --------------------------------------------------------------
   toggleApts (box)
   ----------------------------------------------------------------*/
function toggleApts(box) {
    var apts = document.querySelector('div#airportData').children;
    for(var i=0; i<apts.length; i++)
        apts[i].className = (box.checked)? "revealed": "hidden";
}
