/* basic.js */

"use strict";

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Allow blocks to be hidden or revealed by clicking on a handle
// 1 The block must be wrapped by a div with class="hidden"
// 2 The block must contain <span class="showAlways">Title or similar</span>
// 3 Call 'enableHiddenBlocks()', and the handle will be programmatically 
//   appended to the span named "showAlways".  E.g. specify
//   window.onload = function () {enableHiddenBlocks();}
// The CSS must contain
// 1 div.hidden * {display: none;}
// 2 div.hidden span.showAlways, div.hidden span.showAlways * {display: inline;}
// 3 span.handle {color:red; font-size: smaller; 
//                text-decoration:underline;cursor:pointer;}
function enableHiddenBlocks() {
    // Find all 'hidden' elements, and add handler to the title element
    var elements = document.getElementsByClassName("hidden");
    for(var i=0; i<elements.length; i++) {
        var el = elements[i];
        var title = el.getElementsByClassName("showAlways")[0];
        addRevealHandler(title,el);
    }

    // Repeat the above for all elements that are currently 'revealed'
    var elements = document.getElementsByClassName("revealed");
    for(var i=0; i<elements.length; i++) {
        var el = elements[i];
        var title = el.getElementsByClassName("showAlways")[0];
        addRevealHandler(title,el);
    }

    // Define a handler, specific to each airport, which toggles visibility
    function addRevealHandler(handle,el) {
        handle.onclick = function() {
            if(el.className=="hidden") {
                el.className = "revealed";
                //handle.innerHTML = "&nbsp;Click to hide";
            } else if(el.className=="revealed") {
                el.className = "hidden";
                //handle.innerHTML = "&nbsp;Click to show";
            }
        }
    }
} // enableHiddenBlocks


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function contact( domain, name, subject ) { 
    document.write('<a href="mai' + 
                   'lto:' + name + '@' + domain + '?subject=' + subject + '">' + 
                   name + '@' + domain + '</' + 'a>'); 
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// This function appends section <h1 id="debuglog"> to the page (if 
// it doesn't already exist), and then appends the message.
function debug(msg) {
    // Find the debugging section
    var log = document.getElementById("debuglog");

    // If no "debuglog" element exists, then create one
    if(!log) {
        log = document.createElement("div");
        log.id = "debuglog";
        log.innerHTML = "<h1>Debug Log</h1>";
        document.body.appendChild(log);
    }

    // Now wrap message in a <pre>, and append it to the log
    var pre = document.createElement("pre");
    var text = document.createTextNode(msg+"\n");
    pre.appendChild(text);
    log.appendChild(pre);
} // debug

